# FilterExps 

The `FilterExps.jl` sub-module contains methods to configure filter twin experiments, using a
stored time series as generated by [GenerateTimeSeries](@ref) as the underlying
observation generating process.  The frequency of observations in continuous time is defined
by the frequency of data saved in the time series and is inferred by the experiment
when reading in the data.

Filter experiment configurations are generated by supplying a 
[NamedTuple](https://docs.julialang.org/en/v1/base/base/#Core.NamedTuple)
with the required fields as specified in the experiment method.  Conventions for
these arguments are as follows:
  * `time_series` - the path and file name of the [.jld2](https://juliaio.github.io/JLD2.jl/dev/) truth twin data set;
  * `method` - the sub-method analysis scheme string name;
  * `seed` - the pseudo-random seed that will define, e.g., the observation noise sequence;
  * `nanl` - the number of observation / analysis times to produce a posterior estimate;
  * `obs_un` - the observation error standard deviation, assuming a uniform scaling observation error covariance;
  * `obs_dim` - the dimension of the observation vector;
  * `γ` - defines nonlinearity in the [`DataAssimilationBenchmarks.ObsOperators.alternating_obs_operator`](@ref);
  * `N_ens` - the ensemble size for ensemble-based filters;
  * `s_infl` - the multiplicative inflation for the empirical model state covariance;
  * `p_infl` - the multiplicative inflation for the empirical parameter sample covariance;
  * `p_err` - defines initial parameter sample standard deviation as `p_err` percent of the system parameter value;
  * `p_wlk` - defines the standard deviation of a Gaussian random walk as `p_wlk` percent of the estimated parameter value for a random parameter model.

Standard configurations should be defined in the [SingleExperimentDriver](@ref) module, for reproducing results
and generating standard tests of methods.

## Filter Experiment Methods

```@autodocs
Modules = [DataAssimilationBenchmarks.FilterExps]
```
