# SmootherExps 

The `SmootherExps.jl` sub-module contains methods to configure filter twin experiments,
using a stored time series as generated by [GenerateTimeSeries](@ref) as the underlying
observation generating process.  The frequency of observations in continuous time is defined
by the frequency of data saved in the time series and is inferred by the experiment
when reading in the data.

```@raw html
<div sytle="float:left; width:100%;">
<img style="width:95%" src="https://raw.githubusercontent.com/cgrudz/DataAssimilationBenchmarks.jl/master/assets/cyclingSDA.png" alt="Observation analysis forecast cycle over multiple data assimilation windows">
</div>
```
Smoother experiment configurations are generated by supplying a 
[NamedTuple](https://docs.julialang.org/en/v1/base/base/#Core.NamedTuple)
with the required fields as specified in the experiment method.  Conventions for
these arguments are the same as with the [FilterExps](@ref), with the additional options
that configure  the data assimilation window (DAW) and how this is shifted in time:
  * `lag` - the number of past observation / analysis times to reanalyze in a DAW, corresponding to ``L`` in the figure above;
  * `shift`- the number of observation / analysis times to shift the DAW, corresponding to ``S`` in the figure above;
  * `mda` - (__Multiple Data Assimilation__), type `Bool`, determines whether the technique of multiple data assimilation is used (only compatible with `single_iteration` and `iterative` smoothers.

Currently debugged and validated smoother experiment configurations include
  * `classic_state` - classic ETKS style state estimation
  * `classic_param` - classic ETKS style state-parameter estimation
  * `single_iteration_state` - SIEnKS state estimation
  * `single_iteration_param` - SIEnKS state-parameter estimation
  * `iterative_state` - IEnKS Gauss-Newton style state estimation
  * `iterative_param` - IEnKS Gauss-Newton style state-parameter estimation

Note, the single-iteration and fully iterative Gauss-Newton style smoothers are only defined
for MDA compatible values of lag and shift where the lag is an integer multiple of the shift.

## Smoother Experiment Methods
```@autodocs
Modules = [DataAssimilationBenchmarks.SmootherExps]
```
